import setuptools
import os
import glob


setuptools.setup(
    name='PySubstringSearch',
    version='0.3.1',
    author='Gal Ben David',
    author_email='gal@intsights.com',
    url='https://github.com/Intsights/PySubstringSearch',
    project_urls={
        'Source': 'https://github.com/Intsights/PySubstringSearch',
    },
    license='MIT',
    description='Python library for fast substring/pattern search written in C++ leveraging Suffix Array Algorithm',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords='substring pattern search suffix array c++',
    python_requires='>=3.6',
    zip_safe=False,
    package_data={},
    include_package_data=True,
    ext_modules=[
        setuptools.Extension(
            name='pysubstringsearch',
            sources=glob.glob(
                pathname=os.path.join(
                    'src',
                    'pysubstringsearch.cpp',
                ),
            ),
            language='c++',
            extra_compile_args=[
                '-std=c++17',
            ],
            include_dirs=[
                'src',
            ],
        ),
    ],
)



import unittest
import tempfile

import pysubstringsearch


class PySubstringSearchTestCase(
    unittest.TestCase,
):
    def assert_substring_search(
        self,
        strings,
        substring,
        expected_results,
    ):
        with tempfile.TemporaryDirectory() as tmp_directory:
            writer = pysubstringsearch.Writer(
                index_file_path=f'{tmp_directory}/output.idx',
            )
            for string in strings:
                writer.add_entry(
                    text=string,
                )
            writer.finalize()

            reader = pysubstringsearch.Reader(
                index_file_path=f'{tmp_directory}/output.idx',
            )
            self.assertCountEqual(
                first=reader.search_parallel(
                    substring=substring,
                ),
                second=expected_results,
            )
            self.assertCountEqual(
                first=reader.search_sequential(
                    substring=substring,
                ),
                second=expected_results,
            )

    def test_file_not_found(
        self,
    ):
        with self.assertRaises(
            expected_exception=RuntimeError,
        ):
            pysubstringsearch.Reader(
                index_file_path=f'missing_index_file_path',
            )

    def test_sanity(
        self,
    ):
        strings = [
            'one',
            'two',
            'three',
            'four',
            'five',
            'six',
            'seven',
            'eight',
            'nine',
            'ten',
        ]

        self.assert_substring_search(
            strings=strings,
            substring='four',
            expected_results=[
                'four',
            ],
        )

        self.assert_substring_search(
            strings=strings,
            substring='f',
            expected_results=[
                'four',
                'five',
            ],
        )

        self.assert_substring_search(
            strings=strings,
            substring='our',
            expected_results=[
                'four',
            ],
        )

        self.assert_substring_search(
            strings=strings,
            substring='aaa',
            expected_results=[],
        )

    def test_edgecases(
        self,
    ):
        strings = [
            'one',
            'two',
            'three',
            'four',
            'five',
            'six',
            'seven',
            'eight',
            'nine',
            'ten',
            'tenten',
        ]

        self.assert_substring_search(
            strings=strings,
            substring='none',
            expected_results=[],
        )

        self.assert_substring_search(
            strings=strings,
            substring='one',
            expected_results=[
                'one',
            ],
        )

        self.assert_substring_search(
            strings=strings,
            substring='onet',
            expected_results=[],
        )

        self.assert_substring_search(
            strings=strings,
            substring='ten',
            expected_results=[
                    'ten',
                    'tenten',
                ],
        )

    def test_unicode(
        self,
    ):
        strings = [
            '诶比西',
        ]

        self.assert_substring_search(
            strings=strings,
            substring='诶',
            expected_results=[
                '诶比西',
            ],
        )

        self.assert_substring_search(
            strings=strings,
            substring='诶比',
            expected_results=[
                '诶比西',
            ],
        )

        self.assert_substring_search(
            strings=strings,
            substring='比诶',
            expected_results=[],
        )

        self.assert_substring_search(
            strings=strings,
            substring='none',
            expected_results=[],
        )

    def test_multiple_words_string(
        self,
    ):
        strings = [
            'some short string',
            'another but now a longer string',
            'more text to add',
        ]

        self.assert_substring_search(
            strings=strings,
            substring='short',
            expected_results=[
                'some short string',
            ],
        )
